<?php

if (!defined('TOBBIVM')) header('location:/template/notrepassing.php');
/**
 * This file is part of TobbiVMShop.
 *
 * TobbiVMShop is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TobbiVMShop is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TobbiVMShop.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Datei: shipping.class.php
 *
 *
 * @author Norbert Gebert
 * @copyright Norbert Gebert 25.09.2011
 * @license GPL3
 *
 * @version 0.5b
 * @package Admin
 * @subpackage global
 */
class shipping
{

    // @var float _shipping shipping shippingprice
    private $_shipping;
    // @var float _shippingmax maximum shippingprice
    private $_shippingmax;
    // @var float _shippingfree free shippingprice
    private $_shippingfree;
    // @var string _currency Currency symbol
    private $_currency;

    /**
     * Class-constructor
     *
     * @access	public
     */
    public function __construct()
    {
        $this->_shipping = 0.00;
        $this->_currency = CURRENCY;

        $data = array('config_value');
        db::getInstance()->where('config_key', 'shippingmax');
        $result = db::getInstance()->getOne('config' . SUFFIX, NULL, $data);
        $this->_shippingmax = $result['config_value'];
        $data = array('config_value');
        db::getInstance()->where('config_key', 'shippingfree');
        $result = db::getInstance()->getOne('config' . SUFFIX, NULL, $data);
        $this->_shippingfree = $result['config_value'];
    }

    /**
     * Create shipping value
     *
     * @access	public
     * @param	string	$uploadFilename	File name for the file
     */
    public function calculateShippingCosts($productId, $productValue)
    {
        $data = array('product_shipping_id');
        db::getInstance()->where('product_id', $productId);
        $result = db::getInstance()->getOne('product', NULL, $data);
        db::getInstance()->where('shipping_id', $result['product_shipping_id']);
        $result = db::getInstance()->getOne('shipping');

        if ($result['shipping_value'] <> 0)
        { //Shipping fixed price
            $this->_shipping .= $result['shipping_value'];
            $this->_testCosts();
        } else
        { // Graduated price list
            $text = str_replace("\r\n", "\n", $result['shipping_table']);
            $text = str_replace("\r", "\n", $text);
            $rows = explode("\n", $text);
            foreach ($rows as $row)
            {
                $values = explode(':', $row);
                $number = explode('-', $values[0]);

                if ($productValue >= intval($number[0]) and $productValue <= intval($number[1]))
                {
                    $this->_shipping = $this->_shipping + floatval(str_replace(',',
                                            '.', $values[1]));
                    $this->_testCosts();
                    break;
                }
            }
        }
    }

    /**
     * Check the shipping costs and any correction
     */
    private function _testCosts()
    {
        if ($this->_shipping >= $this->_shippingmax)
                $this->_shipping = $this->_shippingmax;
    }

    /**
     * Get shipping price and check if free shipping
     * 
     * @param bool $currency  TRUE = incl. currency-symbol, FALSE = without
     * @param bool $float     TRUE = return price as float
     */
    public function getShipping($totalPrice, $currency = TRUE, $float = FALSE)
    {
        if ($float == TRUE)
        {
            if ($totalPrice >= $this->_shippingfree) return 0;
            return $this->_shipping;
        } else if ($currency = TRUE)
        {
            if ($totalPrice >= $this->_shippingfree)
                    return number_format((float) 0, 2, ',', '.') . $this->_currency;
            return number_format((float) $this->_shipping, 2, ',', '.') . $this->_currency;
        } else
        {
            if ($totalPrice >= $this->_shippingfree)
                    return number_format((float) 0, 2, ',', '.');
            return number_format((float) $this->_shipping, 2, ',', '.');
        }
    }

}